/*******************************************************************************
 * Copyright (C) 2011 Lars Grammel 
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *    http://www.apache.org/licenses/LICENSE-2.0 
 *     
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.  
 *******************************************************************************/
package org.thechiselgroup.choosel.core.client.error_handling;

import java.util.ArrayList;
import java.util.List;

/**
 * Combines several error handlers. Can be used to e.g. combine logging with
 * showing a feedback dialog.
 * 
 * @author Lars Grammel
 */
public class CompositeErrorHandler implements ErrorHandler {

    private List<ErrorHandler> errorHandlers = new ArrayList<ErrorHandler>();

    public void add(ErrorHandler errorHandler) {
        errorHandlers.add(errorHandler);
    }

    @Override
    public void handleError(Throwable error) {
        for (ErrorHandler errorHandler : errorHandlers) {
            errorHandler.handleError(error);
        }
    }

    public void remove(ErrorHandler errorHandler) {
        errorHandlers.remove(errorHandler);
    }

}
